//xerces includes
#include <xercesc/util/XMLString.hpp>
#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>
#include <xercesc/sax/HandlerBase.hpp>

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/TransService.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>

#include "NCCTreeModel.h"
#include <vector>
#include <string>
#include <iostream>

using namespace std;

//-----------
//constructor
NCCTreeModel::NCCTreeModel()
{
	mModel = NULL;
	mbGetChars = false;
	mbFirstEntry = true;

}

//----------
//destructor
NCCTreeModel::~NCCTreeModel()
{
}

//--------------
//create model function
//start parsing the file
GtkTreeStore* NCCTreeModel::createModel(string nccFile)
{
	const char* cp_file;
	cp_file = nccFile.c_str();
	SAX2XMLReader* parser;

	mNccList.clear();

	mbFirstEntry = true;
	mModel = NULL;

	try
	{
		XMLPlatformUtils::Initialize();
	}
	
	catch (const XMLException& toCatch)
	{
		return NULL;
	}
	
	//create an XMLReader object
	parser = XMLReaderFactory::createXMLReader();
	parser->setContentHandler(this);
	parser->setErrorHandler(this);
	
	parser->setFeature(XMLUni::fgXercesLoadExternalDTD, false);
	parser->setFeature(XMLUni::fgSAX2CoreValidation, false);
	
	
	try
	{
		parser->parse(cp_file);
	}
	catch (const XMLException& toCatch)
	{
		return NULL;

	}

	delete parser;
	XMLPlatformUtils::Terminate();

	return mModel;
}

//--------------
//xerces' start element function
void NCCTreeModel::startElement(const   XMLCh* const    uri,
		const   XMLCh* const    localname,
		const   XMLCh* const    qname,
		const   Attributes&	attributes)
{
	char* node_name = XMLString::transcode(qname);
	bool b_canUse = true;
	char* current_attribute_name;
	char* attribute_value;

	ItemData* p_new_entry;
	int level;

	//just collect headings 1-6 here
	if (strcmp(node_name, "h1") == 0 ||
		strcmp(node_name, "h2") == 0 ||
		strcmp(node_name, "h3") == 0 ||
		strcmp(node_name, "h4") == 0 ||
		strcmp(node_name, "h5") == 0 ||
		strcmp(node_name, "h6") == 0)
	{
		//get the level for this element (1 - 6)
		string tmp_string;
		tmp_string.assign(node_name);
		level = atoi(tmp_string.substr(1, 1).c_str());

		//flag that we want to get the character data
		mbGetChars = true;
	}
	
	else
	{
		//this element is not interesting to us (it's not h1-h6)
		b_canUse = false;
	}

	//if this element is interesting
	if (b_canUse == true)
	{
		//make a new struct
		p_new_entry = new ItemData;

		//-----------------
		//loop to get the id
		//for-loop through the attributes list until we find a match
		for (int i = 0; i<attributes.getLength(); i++)
		{
			current_attribute_name = XMLString::transcode(attributes.getQName(i));
			
			//comparison if statement
			if (strcmp(current_attribute_name, "id") == 0)
			{
				//a match has been found, so save it and break from the loop
				attribute_value = XMLString::transcode(attributes.getValue(i));
				
				p_new_entry->mId.assign(attribute_value);
				XMLString::release(&attribute_value);
				XMLString::release(&current_attribute_name);
				
				break;
			}
			XMLString::release(&current_attribute_name);
		} //end for-loop
		//end getting the id
		//--------------------

		//clear the display text
		p_new_entry->mDisplayText = "";

		//record the level
		p_new_entry->mLevel = level;

		//add to the list
		mNccList.push_back(*p_new_entry);

	}//end if we are going to use this element

	XMLString::release(&node_name);
}

//------------------
//xerces fatal error
void NCCTreeModel::fatalError(const SAXParseException&)
{
	//do something useful with this function later
	cout<<"fatal error!"<<endl;
}

//---------------
//xerces character data
void NCCTreeModel::characters(const XMLCh *const chars, const unsigned int len)
{
	ItemData* p_ncc_data;

	//get a pointer to the last list entry (our current element)
	p_ncc_data = &mNccList[mNccList.size() - 1];

	if (mbGetChars == false)
	{
		return;
	}

	//these characters are intact and are stored properly
	char* elem_chars;
	elem_chars=XMLString::transcode(chars); 

	if (p_ncc_data != NULL && mbGetChars == true)
	{
		p_ncc_data->mDisplayText.assign(elem_chars);
	}

	XMLString::release(&elem_chars);
}

//--------------
//xerces end element function
void NCCTreeModel::endElement( const XMLCh* const uri,
								const XMLCh* const localname,
								const XMLCh* const qname)
{
	char* node_name = XMLString::transcode(qname);

	ItemData* p_ncc_data;

	//get a pointer to the last list entry
	p_ncc_data = &mNccList[mNccList.size() - 1];
	
	//is this element h1-h6?  this should always be true - otherwise it shouldn't even be in the list
	if (strcmp(node_name, "h1") == 0 ||
		strcmp(node_name, "h2") == 0 ||
		strcmp(node_name, "h3") == 0 ||
		strcmp(node_name, "h4") == 0 ||
		strcmp(node_name, "h5") == 0 ||
		strcmp(node_name, "h6") == 0)
	{
		//unflag getChars - we already have the character data for this element
		mbGetChars = false;

		//add the struct to the TreeModel
		addEntryToModel(p_ncc_data);
	}

	XMLString::release(&node_name);
}

//--------------
//function to add a pointer to the tree model
void NCCTreeModel::addEntryToModel(ItemData* pEntry)
{
	//this pointer (pEntry) points to a data structure which is alive throughout the
	//life of this class

	//if this is our first entry in the model, then make a new model
	if (mbFirstEntry == true)
	{
		//tree store data collection
		mModel = gtk_tree_store_new(1, G_TYPE_POINTER);

		mbFirstEntry = false;
			
	}


	if (pEntry == NULL)
	{
		return;
	}

	//find a place to put this entry
	//check its level and use H1-H6 iterators to place it appropriately
	//this part works!

	//whose child is this?
	if (pEntry->mLevel == 1)
	{
		//make an empty row
		gtk_tree_store_append(mModel, &mH1, NULL);

		//add as root
		gtk_tree_store_set(mModel, &mH1, COL_NUM, 
			(gpointer)pEntry, -1);

	}
	
	else if (pEntry->mLevel == 2)
	{
		//make an empty row
		gtk_tree_store_append(mModel, &mH2, &mH1);

		//add as a child of previous
		gtk_tree_store_set(mModel, &mH2, COL_NUM, 
			(gpointer)pEntry, -1);
	}
	
	
	else if (pEntry->mLevel == 3)
	{
		//make an empty row
		gtk_tree_store_append(mModel, &mH3, &mH2);

		//add as a child of previous
		gtk_tree_store_set(mModel, &mH3, COL_NUM, 
			(gpointer)pEntry, -1);
	}
	else if (pEntry->mLevel == 4)
	{
		//make an empty row
		gtk_tree_store_append(mModel, &mH4, &mH3);

		//add as a child of previous
		gtk_tree_store_set(mModel, &mH4, COL_NUM, 
			(gpointer)pEntry, -1);
	}
	else if (pEntry->mLevel == 5)
	{
		//make an empty row
		gtk_tree_store_append(mModel, &mH5, &mH4);

		//add as a child of previous
		gtk_tree_store_set(mModel, &mH5, COL_NUM, 
			(gpointer)pEntry, -1);
	}
	else if (pEntry->mLevel == 6)
	{
		//make an empty row
		gtk_tree_store_append(mModel, &mH6, &mH5);

		//add as a child of previous
		gtk_tree_store_set(mModel, &mH6, COL_NUM,
			(gpointer)pEntry, -1);
	}
	

	else
	{
		//empty
	}


	
}
