#ifndef NCCTREEMODEL_H
#define NCCTREEMODEL_H

#include <string>
#include <vector>
#include <gtk/gtk.h>
#include <xercesc/sax2/DefaultHandler.hpp>

XERCES_CPP_NAMESPACE_USE

XERCES_CPP_NAMESPACE_BEGIN
class Attributes;
XERCES_CPP_NAMESPACE_END

using namespace std;

#define COL_NUM		0

//each item in the tree model points to a struct of type ItemData
struct ItemData
{
	string mDisplayText;
	string mId;
	int mLevel;
};

//NCCTreeModel class
//read in an ncc.html file and collect the headings.  Organize them hierarchically in a tree model.
class NCCTreeModel : public DefaultHandler
{
public:
	NCCTreeModel();
	~NCCTreeModel();

	GtkTreeStore* createModel(string nccFile);
	
	void addEntryToModel(ItemData*);

	//XERCES xml parser functions
	void startElement(const   XMLCh* const    uri,
		const   XMLCh* const    localname,
		const   XMLCh* const    qname,
		const   Attributes&	attributes);

	void fatalError(const SAXParseException&);

	void characters(const XMLCh *const, const unsigned int); 
	void endElement( const XMLCh* const uri,
					const XMLCh* const localname,
					const XMLCh* const qname);

	//this is public so I can check the data integrity during testing
	vector <ItemData> mNccList;

private:
	//the model itself
	GtkTreeStore* mModel;

	//flags
	bool mbFirstEntry;
	bool mbGetChars;

	//iterators for each level (h1 - h6)
	GtkTreeIter mH1, mH2, mH3, mH4, mH5, mH6;

};

#endif